<?php defined('SYSPATH') or die('No direct access allowed.');

/**
 * This class provides invoice item capabilities.
 *
 * @package    OSB
 * @subpackage Invoice
 * @category   Models
 * @author     Deon George
 * @copyright  (c) 2010 Open Source Billing
 * @license    http://dev.osbill.net/license.html
 */
class Model_Invoice_Item extends ORMOSB {
	protected $_updated_column = FALSE; // @todo No update columns

	// Relationships
	protected $_belongs_to = array(
		'product'=>array(),
		'invoice'=>array(),
		'service'=>array()
	);
	protected $_has_one = array(
		'charge'=>array('far_key'=>'charge_id','foreign_key'=>'id')
	);
	protected $_has_many = array(
		'invoice_item_tax'=>array('far_key'=>'id')
	);

	protected $_display_filters = array(
		'date_orig'=>array(
			array('Config::date',array(':value')),
		),
		'date_start'=>array(
			array('Config::date',array(':value')),
		),
		'date_stop'=>array(
			array('Config::date',array(':value')),
		),
	);

	// Display a transaction number
	public function trannum() {
		return sprintf('%03s-%06s',$this->item_type,$this->id);
	}

	// Display the period that a transaction applies
	public function period() {
		if ($this->date_start == $this->date_stop)
			return Config::date($this->date_start);

		else
			return sprintf('%s -> %s',Config::date($this->date_start),Config::date($this->date_stop));
	}

	// Sum up the tax that applies to this invoice item
	public function tax() {
		$result = 0;

		foreach ($this->invoice_item_tax->find_all() as $iit)
			$result += $iit->amount;

		// @todo This shouldnt be required.
		if (! $result)
			$result += round($this->subtotal() *.1,2);

		return $result;
	}

	// This total of this item before discounts and taxes
	public function subtotal() {
		return $this->price_base*$this->quantity;
	}

	// The total of all discounts
	public function discount() {
		return $this->discount_amt;
	}

	public function total() {
		// @todo This rounding should be a system config
		return round($this->subtotal()+$this->tax()-$this->discount(),2);
	}

	public function name() {
		return $this->product_name ? $this->product_name : ($this->item_type == 0 ? _('Service') : _('Other'));
	}

	public function detail() {
		return ($this->item_type == 0 OR $this->quantity == 1) ? HTML::nbsp('') : sprintf('%s@%3.2f',$this->quantity,$this->price_base);
	}

	public function invoice_detail_items() {
		// @todo To fix up this function - needs to be better formed.
		if ($this->item_type == 5)
			return $this->charge->details('invoice_detail_items');

		if ($this->item_type != 0)
			return;

		return $this->service->details('invoice_detail_items');
	}

	public function save(Validation $validation = NULL) {
		// Save the invoice item
		parent::save($validation);

		// Need to save the taxes and discounts associated with the invoice_item
		// @todo This needs to only check if the records have previously been saved, and update them.
		if ($this->saved()) {
			$iito = ORM::factory('invoice_item_tax');

			// Save TAX details
			// @todo tax parameters should come from user session
			foreach (Tax::detail(61,NULL,$this->subtotal()) as $tax) {
				$iito->clear();
				$iito->invoice_item_id = $this->id;
				$iito->tax_id = $tax['id'];
				// @todo Rounding here should come from a global config
				$iito->amount = round($tax['amount'],2);

				if (! $iito->check())
					throw new Kohana_Exception('Couldnt save tax for some reason - failed check()?');

				$iito->save();

				if (! $iito->saved())
					throw new Kohana_Exception('Couldnt save tax for some reason - failed save()?');
			}
		} else
			throw new Kohana_Exception('Couldnt save invoice_item for some reason?');
	}
}
?>
